% Linearisation of model
function [Ac, Bc, Cc, Dc] = linearise_model(x, u, sys)

    % Parameters 
    omega_b = sys.omega_b;
    l_f = sys.l_f;
    r_f = sys.r_f;
    c_f = sys.c_f;
    
    l_g = sys.l_g;
    r_g = sys.r_g;
    
    % Active damping
    k_ad = sys.k_ad;
    omega_ad = sys.omega_ad;
    
    % Current controllers
    kp_c = sys.kp_c;
    ki_c = sys.ki_c;
    
    % Pll controller
    omega_lp_pll = sys.omega_lp_pll;
    kp_pll = sys.kp_pll;
    ki_pll = sys.ki_pll;

    % Measurements
    omega_lp_m = sys.omega_lp_m;

A = [...
                                            0,        omega_b*u(4),                                                                 omega_b/c_f,                                                                  0,                  0,                  0,       -omega_b/c_f,                  0,                  0,                  0,                                                0,                                                0,                    0,                             0, 0, 0, 0, 0;
                                -omega_b*u(4),                   0,                                                                           0,                                                        omega_b/c_f,                  0,                  0,                  0,       -omega_b/c_f,                  0,                  0,                                                0,                                                0,                    0,                             0, 0, 0, 0, 0;
                          -(k_ad*omega_b)/l_f,                   0,                                                 -(omega_b*(kp_c + r_f))/l_f, -(omega_b*(kp_pll*l_f*atan2(x(12), x(11)) + x(13)*ki_pll*l_f))/l_f, (ki_c*omega_b)/l_f,                  0,                  0,                  0, (k_ad*omega_b)/l_f,                  0,  (x(4)*kp_pll*omega_b*x(12))/(x(11)^2 + x(12)^2), -(x(4)*kp_pll*omega_b*x(11))/(x(11)^2 + x(12)^2), -x(4)*ki_pll*omega_b,                             0, 0, 0, 0, 0;
                                            0, -(k_ad*omega_b)/l_f,           (omega_b*(kp_pll*l_f*atan2(x(12), x(11)) + x(13)*ki_pll*l_f))/l_f,                                        -(omega_b*(kp_c + r_f))/l_f,                  0, (ki_c*omega_b)/l_f,                  0,                  0,                  0, (k_ad*omega_b)/l_f, -(x(3)*kp_pll*omega_b*x(12))/(x(11)^2 + x(12)^2),  (x(3)*kp_pll*omega_b*x(11))/(x(11)^2 + x(12)^2),  x(3)*ki_pll*omega_b,                             0, 0, 0, 0, 0;
                                            0,                   0,                                                                          -1,                                                                  0,                  0,                  0,                  0,                  0,                  0,                  0,                                                0,                                                0,                    0,                             0, 0, 0, 0, 0;
                                            0,                   0,                                                                           0,                                                                 -1,                  0,                  0,                  0,                  0,                  0,                  0,                                                0,                                                0,                    0,                             0, 0, 0, 0, 0;
                                  omega_b/l_g,                   0,                                                                           0,                                                                  0,                  0,                  0, -(omega_b*r_g)/l_g,       omega_b*u(4),                  0,                  0,                                                0,                                                0,                    0, (omega_b*u(3)*sin(x(14)))/l_g, 0, 0, 0, 0;
                                            0,         omega_b/l_g,                                                                           0,                                                                  0,                  0,                  0,      -omega_b*u(4), -(omega_b*r_g)/l_g,                  0,                  0,                                                0,                                                0,                    0, (omega_b*u(3)*cos(x(14)))/l_g, 0, 0, 0, 0;
                                     omega_ad,                   0,                                                                           0,                                                                  0,                  0,                  0,                  0,                  0,          -omega_ad,                  0,                                                0,                                                0,                    0,                             0, 0, 0, 0, 0;
                                            0,            omega_ad,                                                                           0,                                                                  0,                  0,                  0,                  0,                  0,                  0,          -omega_ad,                                                0,                                                0,                    0,                             0, 0, 0, 0, 0;
                                 omega_lp_pll,                   0,                                                                           0,                                                                  0,                  0,                  0,                  0,                  0,                  0,                  0,                                    -omega_lp_pll,                                                0,                    0,                             0, 0, 0, 0, 0;
                                            0,        omega_lp_pll,                                                                           0,                                                                  0,                  0,                  0,                  0,                  0,                  0,                  0,                                                0,                                    -omega_lp_pll,                    0,                             0, 0, 0, 0, 0;
                                            0,                   0,                                                                           0,                                                                  0,                  0,                  0,                  0,                  0,                  0,                  0,                       -x(12)/(x(11)^2 + x(12)^2),                        x(11)/(x(11)^2 + x(12)^2),                    0,                             0, 0, 0, 0, 0;
                                            0,                   0,                                                                           0,                                                                  0,                  0,                  0,                  0,                  0,                  0,                  0,      -(kp_pll*omega_b*x(12))/(x(11)^2 + x(12)^2),       (kp_pll*omega_b*x(11))/(x(11)^2 + x(12)^2),       ki_pll*omega_b,                             0, 0, 0, 0, 0;
                              omega_lp_m*x(3),     omega_lp_m*x(4),             omega_lp_m*x(1), omega_lp_m*x(2), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -omega_lp_m, 0, 0, 0;
                             -omega_lp_m*x(4),     omega_lp_m*x(3),             omega_lp_m*x(2), -omega_lp_m*x(1), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -omega_lp_m, 0, 0;
    (omega_lp_m*x(1))/(x(1)^2 + x(2)^2)^(1/2), (omega_lp_m*x(2))/(x(1)^2 + x(2)^2)^(1/2),        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -omega_lp_m, 0;
                                            0, 0, (omega_lp_m*x(3))/(x(3)^2 + x(4)^2)^(1/2),            (omega_lp_m*x(4)/(x(3)^2 + x(4)^2)^(1/2)), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -omega_lp_m];

B = [...
                 0,                  0,                         0,  omega_b*x(2);
                 0,                  0,                         0, -omega_b*x(1);
(kp_c*omega_b)/l_f,                  0,                         0,             0;
                 0, (kp_c*omega_b)/l_f,                         0,             0;
                 1,                  0,                         0,             0;
                 0,                  1,                         0,             0;
                 0,                  0, -(omega_b*cos(x(14)))/l_g,  x(8)*omega_b;
                 0,                  0,  (omega_b*sin(x(14)))/l_g, -x(7)*omega_b;
                 0,                  0,                         0,             0;
                 0,                  0,                         0,             0;
                 0,                  0,                         0,             0;
                 0,                  0,                         0,             0;
                 0,                  0,                         0,             0;
                 0,                  0,                         0,             0;
                 0,                  0,                         0,             0;
                 0,                  0,                         0,             0;
                 0,                  0,                         0,             0;
                 0,                  0,                         0,             0];


C = [...
                           1,                            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
                           0,                            1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
                           0,                            0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
                           0,                            0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
                           0,                            0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
                           0,                            0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
                           0,                            0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
                           0,                            0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
                           0,                            0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0;
                           0,                            0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0;
                           0,                            0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0;
                           0,                            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0;
                           0,                            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0;
                           0,                            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0;
                           0,                            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0;
                           0,                            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0;
                           0,                            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0;
                           0,                            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1];




Ac = A;
Bc = B;
Cc = C;
ny = size(Cc,1);
nu = size(Bc,2);
Dc =  zeros(ny,nu);
end