%nonlin_eq.m
function dx = nonlin_eq(x,u,sys)
    %Nonlinear model of voltage source converter without outer controller
    
    % Parameters 
    omega_b = sys.omega_b;
    l_f = sys.l_f;
    r_f = sys.r_f;
    c_f = sys.c_f;
    l_g = sys.l_g;
    r_g = sys.r_g;
 
    % Active damping
    k_ad = sys.k_ad;
    omega_ad = sys.omega_ad;
    
    % Current controllers
    kp_c = sys.kp_c;
    ki_c = sys.ki_c;
    
    % Pll controller
    omega_lp_pll = sys.omega_lp_pll;
    kp_pll = sys.kp_pll;
    ki_pll = sys.ki_pll;
    
    % measurements
    omega_lp_m = sys.omega_lp_m;
    dx = zeros(18,1);
    dx(1) = (omega_b*(x(3) - x(7) + c_f*u(4)*x(2)))/c_f;
    dx(2) = -(omega_b*(x(8) - x(4) + c_f*u(4)*x(1)))/c_f;
    dx(3) = -(omega_b*(x(3)*kp_c - x(5)*ki_c - u(1)*kp_c + x(3)*r_f - k_ad*x(9) + k_ad*x(1) + x(13)*x(4)*ki_pll*l_f + x(4)*kp_pll*l_f*atan2(x(12), x(11))))/l_f;
    dx(4) = (omega_b*(x(6)*ki_c - x(4)*kp_c + u(2)*kp_c - x(4)*r_f + k_ad*x(10) - k_ad*x(2) + x(13)*x(3)*ki_pll*l_f + x(3)*kp_pll*l_f*atan2(x(12), x(11))))/l_f;
    dx(5) = u(1) - x(3);
    dx(6) = u(2) - x(4);
    dx(7) = (omega_b*(x(1) - x(7)*r_g - u(3)*cos(x(14)) + x(8)*l_g*u(4)))/l_g;
    dx(8) = (omega_b*(x(2) - x(8)*r_g + u(3)*sin(x(14)) - x(7)*l_g*u(4)))/l_g;
    dx(9) = -omega_ad*(x(9) - x(1));
    dx(10) = -omega_ad*(x(10) - x(2));
    dx(11) = omega_lp_pll*(x(1) - x(11));
    dx(12) = omega_lp_pll*(x(2) - x(12));
    dx(13) = atan2(x(12), x(11));
    dx(14) = omega_b*(x(13)*ki_pll + kp_pll*atan2(x(12), x(11)));
    dx(15) = omega_lp_m*(x(1)*x(3) + x(2)*x(4) - x(15));
    dx(16) = omega_lp_m*(-x(1)*x(4) + x(2)*x(3) - x(16));
    dx(17) = omega_lp_m*((x(1)^2 + x(2)^2)^(1/2) - x(17));
    dx(18) = omega_lp_m*((x(3)^2 + x(4)^2)^(1/2) - x(18));
end