%% IMPORTANT PARAMETERS

Ts.mpc = 1e-3;              % Step size for MPC
Ts.simulation = 1e-4;       % Step size
mpc_par.Ph = 5;              % Prediction Horizon
mpc_par.Mh = mpc_par.Ph;          % Control Horizon
mpc_par.states.nx = 18;        % Number of states 
mpc_par.states.nu = 4;         % Number of inputs
mpc_par.states.ny = 18;        % Number of outputs

sys = set_system_parameters;
parametersBatterySystem
strong_grid = false;            % Define Grid to simulate
mpc_par.runAdaptiveMPC = true;  % Define MPC to run
mpc_par.turnOnTime = .5;        % Turn on time for MPC
mpc_par.bess_placement = 9;     % BESS placement
%% Load Grid data
grid = calc_grid_data(mpc_par.bess_placement, conv, strong_grid);     % Calculate grid data
sys = set_system_parameters(imag(grid.z_th), real(grid.z_th), grid.v_th);          %Update system parameters with grid data.
parametersBatterySystem;
%initial state and input values
mpc_par.states.x0 = [1 0 1 0 0 0 0 0 1 0 1 0 0 0 0 0 1 0]';     % Set initial state values
mpc_par.states.u0 = [0 0 grid.v_th 1]';                 % Set initial input values

% Find equilibrium point for linearization
mpc_par.model = @(x)nonlin_eq(x, mpc_par.states.u0, sys);
[mpc_par.states.xInit, fval] = fsolve(mpc_par.model,mpc_par.states.x0);

mpc_par.states.uInit = mpc_par.states.u0;
mpc_par.states.MVInit = mpc_par.states.uInit(1:(2));
mpc_par.states.MDInit = mpc_par.states.uInit(3:(end));

% Weighting matrices
mpc_par.weights.Ru = [0.001 0.001];           %  Weights of control inputs
mpc_par.weights.Ru = diag(mpc_par.weights.Ru);
mpc_par.weights.RuRate = [1 1];           % Weights of control input rates
mpc_par.weights.RuRate = diag(mpc_par.weights.RuRate);
mpc_par.weights.Qy = [0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 .2 50 0]; % Weights of output variables
mpc_par.weights.Qy = diag(mpc_par.weights.Qy);
mpc_par.weights.ECR = 1e4;

% Reference values
mpc_par.states.r = [0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 mpc_par.states.xInit(17) 0];
% mpc_par.states.r = [0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0];
% Plant model initializiation
[Ac, Bc, Cc, Dc] = linearise_model(mpc_par.states.xInit, mpc_par.states.uInit, sys);
[Ad, Bd, Cd, Dd] = discretize(Ac, Bc, Cc, Dc, Ts.mpc);
mpc_par.plant = ss(Ad, Bd, Cd, Dd, Ts.mpc);
mpc_par.states.yInit = Cc * mpc_par.states.xInit;

% set names for variables
mpc_par.plant.InputName = {'i_{cv,ref,d}', 'i_{cv,ref,q}', 'v_g_amp', 'omega_g'};  % set names of input variables
mpc_par.plant.StateName = {'v_o_d', 'v_o_q', 'i_cv_d', 'i_cv_q', 'gamma_d', 'gamma_q', 'i_o_d', 'i_o_q',...
                   'phi_d', 'phi_q', 'v_pll_    d', 'v_pll_q', 'epsilon_pll', 'delta_theta_pll', 'p_o_f', 'q_o_f', 'v_o_f_amp', 'i_cv_f_amp'};      % set names of state variables
mpc_par.plant.OutputName = mpc_par.plant.Statename; % set names of output variables

mpc_par.plant=setmpcsignals(mpc_par.plant,'MV',[1 2],'MD',[3 4]);
old_status = mpcverbosity('on');

% Constraints for manipulated variables
mpc_par.constraints.uMin = {-1, -1};
mpc_par.constraints.uMax = {1, 1};
mpc_par.constraints.uRateMin = {-1, -1};
mpc_par.constraints.uRateMax = {1, 1};

% Constraints for state and output variables
mpc_par.constraints.xMin = {-inf -inf -inf -inf -inf -inf -inf -inf -inf -inf -inf -inf -inf -inf -1.1 -1.1 .92 -1.1};
mpc_par.constraints.xMax = {inf inf inf inf inf inf inf inf inf inf inf inf inf inf 1.1 1.1 1.08 1.1 };
mpc_par.constraints.yMin = mpc_par.constraints.xMin;
mpc_par.constraints.yMax = mpc_par.constraints.xMax;

% Define linear mpc parameters
mpcobj = mpc(mpc_par.plant, Ts.mpc, mpc_par.Ph, mpc_par.Mh);
mpcobj.MV = struct('Min', mpc_par.constraints.uMin, 'Max', mpc_par.constraints.uMax, 'RateMin', mpc_par.constraints.uRateMin, 'RateMax', mpc_par.constraints.uRateMax);
mpcobj.OutputVariables = struct('Min', mpc_par.constraints.yMin, 'Max', mpc_par.constraints.yMax);
    % Tuning Weights
mpcobj.W.OutputVariables = {mpc_par.weights.Qy};
mpcobj.W.ManipulatedVariables = {mpc_par.weights.Ru};
mpcobj.W.ManipulatedVariablesRate = {mpc_par.weights.RuRate};
mpcobj.W.ECR = mpc_par.weights.ECR;

% Initial state and input values for mpc
mpcobj.Model.Nominal = struct('X', mpc_par.states.xInit, 'U', [mpc_par.states.MVInit; mpc_par.states.MDInit], 'Y', mpc_par.states.yInit, 'DX', zeros(mpc_par.states.nx,1));
mpc_par.xmpc = mpcstate(mpcobj);
mpc_par.states.xInit

xInit = mpc_par.states.xInit;
uInit = mpc_par.states.uInit;
yInit = mpc_par.states.yInit;
